% Copyright (c) 2016, NVIDIA CORPORATION. All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions
% are met:
%  * Redistributions of source code must retain the above copyright
%    notice, this list of conditions and the following disclaimer.
%  * Redistributions in binary form must reproduce the above copyright
%    notice, this list of conditions and the following disclaimer in the
%    documentation and/or other materials provided with the distribution.
%  * Neither the name of NVIDIA CORPORATION nor the names of its
%    contributors may be used to endorse or promote products derived
%    from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
% EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
% OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%
%Description: Compute monomial basis for the Krylov subspace 
%             Y = [x,(A*inv(M))^1 *x,(A*inv(M))^2 *x,...,(A*inv(M))^(s-1) *x]
%             and preconditioned basis
%             Z = inv(M)*[x,(A*inv(M))^1 *x,(A*inv(M))^2 *x,...,(A*inv(M))^(s-2) *x]
%Input Parameters:
%A - matrix
%M - preconditioner M=LU => inv(M) = inv(U)*inv(L)
%x - initial vector (or a set of vectors)
%s - dimension of the Krylov subspace
%Output Parameters:
%Y - resulting monomial basis [x,(A*inv(M))^1 *x,(A*inv(M))^2 *x,...,(A*inv(M))^(s-1) *x]
%Z - resulting preconditioned monomial basis, with one dimension less


function  [Y,Z] = s_lumatvecs(A,L,U,x,s)
%x might be a tall matrix
[m,n]=size(x); 
%pre-allocation for speed
Y=zeros(m,s*n);
Z=zeros(m,(s-1)*n);

%main loop
Y(:,1:n) = x;   
for i=2:s
    o = (i-1)*n;
    z = U\(L\Y(:,1+o-n:o)); %  inv(M)*x
    y = A*z;                %A*inv(M)*x
    Z(:,1+o-n:o) = z;
    Y(:,1+o:o+n) = y;
end
